@@ -99,7 +99,9 @@ typedef enum
 	RHYTHMDB_PROP_ARTIST,
 	RHYTHMDB_PROP_ALBUM,
 	RHYTHMDB_PROP_TRACK_NUMBER,
+	RHYTHMDB_PROP_TRACK_TOTAL,
 	RHYTHMDB_PROP_DISC_NUMBER,
+	RHYTHMDB_PROP_DISC_TOTAL,
 	RHYTHMDB_PROP_DURATION,
 	RHYTHMDB_PROP_FILE_SIZE,
 	RHYTHMDB_PROP_LOCATION,
@@ -140,11 +142,12 @@ typedef enum
 	RHYTHMDB_PROP_STATUS,
 	RHYTHMDB_PROP_DESCRIPTION,
 	RHYTHMDB_PROP_SUBTITLE,
-	RHYTHMDB_PROP_SUMMARY,
+	RHYTHMDB_PROP_SUMMARY,				/* obsolete */
 	RHYTHMDB_PROP_LANG,
 	RHYTHMDB_PROP_COPYRIGHT,
 	RHYTHMDB_PROP_IMAGE,
 	RHYTHMDB_PROP_POST_TIME,
+	RHYTHMDB_PROP_PODCAST_GUID,
 
 	RHYTHMDB_PROP_MUSICBRAINZ_TRACKID,
 	RHYTHMDB_PROP_MUSICBRAINZ_ARTISTID,
@@ -178,10 +181,6 @@ typedef enum
 } RhythmDBPropType;
 
 enum {
-	RHYTHMDB_PODCAST_FEED_STATUS_HIDDEN = 0,
-	RHYTHMDB_PODCAST_FEED_STATUS_NORMAL = 1,
-	RHYTHMDB_PODCAST_FEED_STATUS_UPDATING = 2,
-
 	RHYTHMDB_PODCAST_STATUS_COMPLETE = 100,
 	RHYTHMDB_PODCAST_STATUS_ERROR = 101,
 	RHYTHMDB_PODCAST_STATUS_WAITING = 102,
@@ -195,6 +194,8 @@ enum {
 #define RHYTHMDB_PROP_COVER_ART			"rb:coverArt"
 #define RHYTHMDB_PROP_COVER_ART_URI		"rb:coverArt-uri"
 
+typedef void (*RhythmDBEntryForeachFunc) (RhythmDBEntry *entry, gpointer data);
+
 GType rhythmdb_query_type_get_type (void);
 GType rhythmdb_prop_type_get_type (void);
 
@@ -288,11 +289,11 @@ struct _RhythmDBClass
 
 	gboolean 	(*impl_evaluate_query)	(RhythmDB *db, RhythmDBQuery *query, RhythmDBEntry *entry);
 
-	void		(*impl_entry_foreach)	(RhythmDB *db, GFunc func, gpointer data);
+	void		(*impl_entry_foreach)	(RhythmDB *db, RhythmDBEntryForeachFunc func, gpointer data);
 
 	gint64		(*impl_entry_count)	(RhythmDB *db);
 
-	void		(*impl_entry_foreach_by_type) (RhythmDB *db, RhythmDBEntryType *type, GFunc func, gpointer data);
+	void		(*impl_entry_foreach_by_type) (RhythmDB *db, RhythmDBEntryType *type, RhythmDBEntryForeachFunc func, gpointer data);
 
 	gint64		(*impl_entry_count_by_type) (RhythmDB *db, RhythmDBEntryType *type);
 
@@ -364,13 +365,13 @@ gboolean	rhythmdb_evaluate_query		(RhythmDB *db, RhythmDBQuery *query,
 						 RhythmDBEntry *entry);
 
 void		rhythmdb_entry_foreach		(RhythmDB *db,
-						 GFunc func,
+						 RhythmDBEntryForeachFunc func,
 						 gpointer data);
 gint64		rhythmdb_entry_count		(RhythmDB *db);
 
 void		rhythmdb_entry_foreach_by_type  (RhythmDB *db,
 						 RhythmDBEntryType *entry_type,
-						 GFunc func,
+						 RhythmDBEntryForeachFunc func,
 						 gpointer data);
 gint64		rhythmdb_entry_count_by_type	(RhythmDB *db,
 						 RhythmDBEntryType *entry_type);
