@@ -261,9 +261,6 @@ playlist_load_started_cb (TotemPlParser *parser, const char *uri, GHashTable *me
 	rb_debug ("loading new playlist %s", uri);
 
 	title = g_hash_table_lookup (metadata, TOTEM_PL_PARSER_FIELD_TITLE);
-	if (title == NULL)
-		title = _("Unnamed playlist");
-
 	mgr->priv->loading_playlist =
 			RB_STATIC_PLAYLIST_SOURCE (rb_playlist_manager_new_playlist (mgr, title, FALSE));
 }
@@ -1732,7 +1729,6 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 	GApplication *app;
 	gboolean playlist_active;
 	gboolean playlist_local = FALSE;
-	gboolean party_mode;
 	gboolean can_save;
 	gboolean can_edit;
 	gboolean can_rename;
@@ -1741,8 +1737,6 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 
 	app = g_application_get_default ();
 
-	party_mode = rb_shell_get_party_mode (mgr->priv->shell);
-
 	if (mgr->priv->selected_source != NULL) {
 		g_object_unref (mgr->priv->selected_source);
 	}
@@ -1753,12 +1747,11 @@ rb_playlist_manager_set_source (RBPlaylistManager *mgr,
 		g_object_get (mgr->priv->selected_source, "is-local", &playlist_local, NULL);
 	}
 
-	can_save = playlist_local && !party_mode;
+	can_save = playlist_local;
 	gaction = g_action_map_lookup_action (G_ACTION_MAP (app), "playlist-save");
 	g_object_set (gaction, "enabled", can_save, NULL);
 
-	can_edit = (playlist_local && RB_IS_AUTO_PLAYLIST_SOURCE (mgr->priv->selected_source) &&
-		    !party_mode);
+	can_edit = (playlist_local && RB_IS_AUTO_PLAYLIST_SOURCE (mgr->priv->selected_source));
 	gaction = g_action_map_lookup_action (G_ACTION_MAP (app), "playlist-edit");
 	g_object_set (gaction, "enabled", can_edit, NULL);
 
