@@ -45,7 +45,6 @@
 
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
-#include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <girepository.h>
 
@@ -62,12 +61,7 @@
 #include "rb-shell.h"
 #include "rb-debug.h"
 #include "rb-dialog.h"
-#ifdef WITH_RHYTHMDB_TREE
 #include "rhythmdb-tree.h"
-#else
-#error "no database specified. configure broken?"
-#endif
-#include "rb-stock-icons.h"
 #include "rb-display-page-tree.h"
 #include "rb-display-page-group.h"
 #include "rb-file-helpers.h"
@@ -77,7 +71,6 @@
 #include "rb-track-transfer-queue.h"
 #include "rb-shell-clipboard.h"
 #include "rb-shell-player.h"
-#include "rb-statusbar.h"
 #include "rb-shell-preferences.h"
 #include "rb-library-source.h"
 #include "rb-podcast-source.h"
@@ -91,7 +84,6 @@
 #include "rb-util.h"
 #include "rb-display-page-model.h"
 #include "rb-song-info.h"
-#include "rb-marshal.h"
 #include "rb-missing-plugins.h"
 #include "rb-header.h"
 #include "rb-podcast-manager.h"
@@ -101,11 +93,14 @@
 #include "rb-auto-playlist-source.h"
 #include "rb-builder-helpers.h"
 #include "rb-display-page-menu.h"
-
-#define UNINSTALLED_PLUGINS_LOCATION "plugins"
+#include "rb-list-model.h"
+#include "rb-task-list.h"
+#include "rb-task-list-display.h"
 
 #define PLAYING_ENTRY_NOTIFY_TIME 4
 
+#define ALBUM_ART_MIN_SIZE	32
+
 static void rb_shell_class_init (RBShellClass *klass);
 static void rb_shell_init (RBShell *shell);
 static void rb_shell_constructed (GObject *object);
@@ -133,7 +128,6 @@ static gboolean rb_shell_key_press_event_cb (GtkWidget *win,
 					     RBShell *shell);
 static void rb_shell_sync_window_state (RBShell *shell, gboolean dont_maximise);
 static void rb_shell_sync_paned (RBShell *shell);
-static void rb_shell_sync_party_mode (RBShell *shell);
 static void rb_shell_select_page (RBShell *shell, RBDisplayPage *display_page);
 static void display_page_selected_cb (RBDisplayPageTree *display_page_tree,
 				      RBDisplayPage *page,
@@ -157,10 +151,10 @@ static void rb_shell_player_window_title_changed_cb (RBShellPlayer *player,
 					             const char *window_title,
 					             RBShell *shell);
 static void rb_shell_playing_changed_cb (RBShellPlayer *player, gboolean playing, RBShell *shell);
+static void rb_shell_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry, RBShell *shell);
+static void rb_shell_settings_changed_cb (GSettings *settings, const char *key, RBShell *shell);
 
-static void rb_shell_jump_to_current (RBShell *shell);
-static void rb_shell_jump_to_entry_with_source (RBShell *shell, RBSource *source,
-						RhythmDBEntry *entry);
+static void rb_shell_jump_to_current (RBShell *shell, gboolean select_page);
 static void rb_shell_play_entry (RBShell *shell, RhythmDBEntry *entry);
 static void rb_shell_load_complete_cb (RhythmDB *db, RBShell *shell);
 static void rb_shell_set_visibility (RBShell *shell,
@@ -177,7 +171,6 @@ static void paned_size_allocate_cb (GtkWidget *widget,
 
 static void jump_to_playing_action_cb (GSimpleAction *, GVariant *, gpointer);
 static void add_music_action_cb (GAction *action, GVariant *parameter, RBShell *shell);
-static void view_party_mode_changed_cb (GAction *action, GVariant *parameter, RBShell *shell);
 
 static gboolean rb_shell_visibility_changing (RBShell *shell, gboolean initial, gboolean visible);
 
@@ -207,7 +200,8 @@ enum
 	PROP_VISIBILITY,
 	PROP_TRACK_TRANSFER_QUEUE,
 	PROP_AUTOSTARTED,
-	PROP_DISABLE_PLUGINS
+	PROP_DISABLE_PLUGINS,
+	PROP_TASK_LIST
 };
 
 enum
@@ -272,11 +266,11 @@ struct _RBShellPrivate
 	RBShellPlayer *player_shell;
 	RBShellClipboard *clipboard_shell;
 	RBHeader *header;
-	RBStatusbar *statusbar;
 	RBPlaylistManager *playlist_manager;
 	RBRemovableMediaManager *removable_media_manager;
 	RBTrackTransferQueue *track_transfer_queue;
 	RBPodcastManager *podcast_manager;
+	GtkWidget *task_list_display;
 
 	RBLibrarySource *library_source;
 	RBPodcastSource *podcast_source;
@@ -292,13 +286,13 @@ struct _RBShellPrivate
 	char *cached_title;
 	gboolean cached_playing;
 
-	gboolean party_mode;
-
 	GSettings *settings;
 
 	GSettings *plugin_settings;
 	PeasEngine *plugin_engine;
 	PeasExtensionSet *activatable;
+
+	RBTaskList *task_list;
 };
 
 static GMountOperation *
@@ -311,6 +305,19 @@ rb_shell_create_mount_op_cb (RhythmDB *db, RBShell *shell)
 	return op;
 }
 
+static gboolean
+accept_art_pixbuf (GdkPixbuf *pixbuf)
+{
+	if ((gdk_pixbuf_get_width (pixbuf) < ALBUM_ART_MIN_SIZE) ||
+	    (gdk_pixbuf_get_height (pixbuf) < ALBUM_ART_MIN_SIZE)) {
+		rb_debug ("rejecting too small (%dx%d) image",
+			  gdk_pixbuf_get_width (pixbuf),
+			  gdk_pixbuf_get_height (pixbuf));
+		return FALSE;
+	}
+	return TRUE;
+}
+
 static GValue *
 load_external_art_cb (RBExtDB *store, GValue *value, RBShell *shell)
 {
@@ -332,6 +339,9 @@ load_external_art_cb (RBExtDB *store, GValue *value, RBShell *shell)
 		GByteArray *bytes = g_value_get_boxed (value);
 		data = (const char *)bytes->data;
 		data_size = bytes->len;
+	} else if (G_VALUE_HOLDS (value, G_TYPE_BYTES)) {
+		GBytes *bytes = g_value_get_boxed (value);
+		data = g_bytes_get_data (bytes, &data_size);
 	} else {
 		rb_debug ("unable to load pixbufs from values of type %s", G_VALUE_TYPE_NAME (value));
 		return NULL;
@@ -355,6 +365,11 @@ load_external_art_cb (RBExtDB *store, GValue *value, RBShell *shell)
 	}
 
 	pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
+	if (accept_art_pixbuf (pixbuf) == FALSE) {
+		g_object_unref (loader);
+		return NULL;
+	}
+
 	v = g_new0 (GValue, 1);
 	g_value_init (v, GDK_TYPE_PIXBUF);
 	g_value_set_object (v, pixbuf);
@@ -388,6 +403,9 @@ store_external_art_cb (RBExtDB *store, GValue *value, RBShell *shell)
 	}
 
 	pixbuf = GDK_PIXBUF (g_value_get_object (value));
+	if (accept_art_pixbuf (pixbuf) == FALSE) {
+		return NULL;
+	}
 
 	/* switch to png if the image has an alpha channel */
 	if (gdk_pixbuf_get_has_alpha (pixbuf)) {
@@ -431,11 +449,7 @@ construct_db (RBShell *shell)
 		pathname = rb_find_user_data_file ("rhythmdb.xml");
 	}
 
-#ifdef WITH_RHYTHMDB_TREE
 	shell->priv->db = rhythmdb_tree_new (pathname);
-#elif defined(WITH_RHYTHMDB_GDA)
-	shell->priv->db = rhythmdb_gda_new (pathname);
-#endif
 	g_free (pathname);
 
 	if (shell->priv->dry_run)
@@ -478,6 +492,7 @@ static void
 construct_widgets (RBShell *shell)
 {
 	GtkWindow *win;
+	GtkStyleContext *context;
 
 	rb_profile_start ("constructing widgets");
 
@@ -506,6 +521,10 @@ construct_widgets (RBShell *shell)
 
 	rb_debug ("shell: initializing shell services");
 
+	shell->priv->task_list = rb_task_list_new ();
+	shell->priv->task_list_display = rb_task_list_display_new (rb_task_list_get_model (shell->priv->task_list));
+	gtk_widget_show (shell->priv->task_list_display);
+
 	shell->priv->podcast_manager = rb_podcast_manager_new (shell->priv->db);
 	shell->priv->track_transfer_queue = rb_track_transfer_queue_new (shell);
 	shell->priv->accel_group = gtk_accel_group_new ();
@@ -528,6 +547,10 @@ construct_widgets (RBShell *shell)
 				 "playing-changed",
 				 G_CALLBACK (rb_shell_playing_changed_cb),
 				 shell, 0);
+	g_signal_connect_object (shell->priv->player_shell,
+				 "playing-song-changed",
+				 G_CALLBACK (rb_shell_playing_song_changed_cb),
+				 shell, 0);
 	shell->priv->clipboard_shell = rb_shell_clipboard_new (shell->priv->db);
 
 	shell->priv->display_page_tree = rb_display_page_tree_new (shell);
@@ -543,10 +566,6 @@ construct_widgets (RBShell *shell)
 	gtk_widget_show (GTK_WIDGET (shell->priv->header));
 	g_settings_bind (shell->priv->settings, "time-display", shell->priv->header, "show-remaining", G_SETTINGS_BIND_DEFAULT);
 
-	shell->priv->statusbar = rb_statusbar_new (shell->priv->db,
-						   shell->priv->track_transfer_queue);
-	gtk_widget_show (GTK_WIDGET (shell->priv->statusbar));
-
 	g_signal_connect_object (shell->priv->display_page_tree, "selected",
 				 G_CALLBACK (display_page_selected_cb), shell, 0);
 
@@ -576,6 +595,8 @@ construct_widgets (RBShell *shell)
 
 	/* set up sidebars */
 	shell->priv->paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
+	context = gtk_widget_get_style_context (shell->priv->paned);
+	gtk_style_context_add_class (context, "sidebar-paned");
 	shell->priv->right_paned = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 	gtk_widget_show_all (shell->priv->right_paned);
 	g_signal_connect_object (G_OBJECT (shell->priv->right_paned),
@@ -604,6 +625,9 @@ construct_widgets (RBShell *shell)
 		gtk_box_pack_start (GTK_BOX (vbox2),
 				    GTK_WIDGET (shell->priv->bottom_container),
 				    FALSE, FALSE, 0);
+		gtk_box_pack_start (GTK_BOX (vbox2),
+				    GTK_WIDGET (shell->priv->task_list_display),
+				    FALSE, FALSE, 0);
 
 		gtk_paned_pack1 (GTK_PANED (shell->priv->right_paned),
 				 vbox2, TRUE, TRUE);
@@ -635,7 +659,6 @@ construct_widgets (RBShell *shell)
 
 	gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), GTK_WIDGET (shell->priv->top_container), FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), shell->priv->paned, TRUE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), GTK_WIDGET (shell->priv->statusbar), FALSE, TRUE, 0);
 	gtk_widget_show_all (shell->priv->main_vbox);
 
 	gtk_container_add (GTK_CONTAINER (win), shell->priv->main_vbox);
@@ -681,7 +704,6 @@ static void
 construct_load_ui (RBShell *shell)
 {
 	GApplication *app = g_application_get_default ();
-	gboolean shell_shows_app_menu;
 	GtkWidget *toolbar;
 	GtkBuilder *builder;
 	GtkToolItem *tool_item;
@@ -697,6 +719,15 @@ construct_load_ui (RBShell *shell)
 
 	shell->priv->play_button = GTK_WIDGET (gtk_builder_get_object (builder, "play-button"));
 
+	image = gtk_button_get_image (GTK_BUTTON (gtk_builder_get_object (builder, "next-button")));
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), "media-skip-forward-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+
+	image = gtk_button_get_image (GTK_BUTTON (gtk_builder_get_object (builder, "previous-button")));
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), "media-skip-backward-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+
+	image = gtk_button_get_image (GTK_BUTTON (gtk_builder_get_object (builder, "play-button")));
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), "media-playback-start-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+
 	/* this seems a bit unnecessary */
 	gtk_actionable_set_action_target_value (GTK_ACTIONABLE (gtk_builder_get_object (builder, "shuffle-button")),
 						g_variant_new_boolean (TRUE));
@@ -717,27 +748,22 @@ construct_load_ui (RBShell *shell)
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);
 
 
-	/* menu tool button, only shown if the shell doesn't show the app menu,
-	 * or in party mode where the app menu is inaccessible.
-	 */
+	/* menu tool button */
 	menu_button = gtk_menu_button_new ();
 	model = rb_application_get_shared_menu (RB_APPLICATION (app), "app-menu");
 	gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (menu_button), model);
+	gtk_style_context_add_class (gtk_widget_get_style_context (menu_button), GTK_STYLE_CLASS_RAISED);
+	gtk_widget_set_valign (menu_button, GTK_ALIGN_CENTER);
 
-	g_object_get (gtk_settings_get_default (),
-		      "gtk-shell-shows-app-menu", &shell_shows_app_menu,
-		      NULL);
-	if (shell_shows_app_menu == FALSE) {
-		gtk_widget_add_accelerator (menu_button,
-					    "activate",
-					    shell->priv->accel_group,
-					    GDK_KEY_F10,
-					    0,
-					    GTK_ACCEL_VISIBLE);
-		rb_application_set_menu_accelerators (shell->priv->application, model, TRUE);
-	}
+	gtk_widget_add_accelerator (menu_button,
+				    "activate",
+				    shell->priv->accel_group,
+				    GDK_KEY_F10,
+				    0,
+				    GTK_ACCEL_VISIBLE);
+	rb_application_set_menu_accelerators (shell->priv->application, model, TRUE);
 
-	image = gtk_image_new_from_icon_name ("emblem-system-symbolic", GTK_ICON_SIZE_LARGE_TOOLBAR);
+	image = gtk_image_new_from_icon_name ("open-menu-symbolic", GTK_ICON_SIZE_SMALL_TOOLBAR);
 	gtk_container_add (GTK_CONTAINER (menu_button), image);
 
 	shell->priv->menu_button = GTK_WIDGET (gtk_tool_item_new ());
@@ -745,6 +771,8 @@ construct_load_ui (RBShell *shell)
 	gtk_widget_show_all (shell->priv->menu_button);
 	gtk_toolbar_insert (GTK_TOOLBAR (toolbar), GTK_TOOL_ITEM (shell->priv->menu_button), -1);
 
+	rb_application_add_accelerator (RB_APPLICATION (app), "<Primary>q", "app.quit", NULL);
+
 	rb_profile_end ("loading ui");
 }
 
@@ -778,12 +806,14 @@ construct_plugins (RBShell *shell)
 		return;
 	}
 
+	g_type_ensure (PEAS_GTK_TYPE_PLUGIN_MANAGER);
+
 	rb_profile_start ("loading plugins");
 	shell->priv->plugin_settings = g_settings_new ("org.gnome.rhythmbox.plugins");
 
 	shell->priv->plugin_engine = peas_engine_new ();
 	/* need an #ifdef for this? */
-	peas_engine_enable_loader (shell->priv->plugin_engine, "python");
+	peas_engine_enable_loader (shell->priv->plugin_engine, "python3");
 
 	typelib_dir = g_build_filename (LIBDIR,
 					"girepository-1.0",
@@ -833,14 +863,6 @@ construct_plugins (RBShell *shell)
 	g_free (plugindir);
 	g_free (plugindatadir);
 
-#ifdef USE_UNINSTALLED_DIRS
-	plugindir = g_build_filename (SHARE_UNINSTALLED_BUILDDIR, "..", UNINSTALLED_PLUGINS_LOCATION, NULL);
-	rb_debug ("plugin search path: %s", plugindir);
-	peas_engine_add_search_path (shell->priv->plugin_engine,
-				     plugindir,
-				     plugindir);
-	g_free (plugindir);
-#endif
 
 	shell->priv->activatable = peas_extension_set_new (shell->priv->plugin_engine,
 							   PEAS_TYPE_ACTIVATABLE,
@@ -928,10 +950,7 @@ construct_plugins (RBShell *shell)
 static gboolean
 _scan_idle (RBShell *shell)
 {
-
-	GDK_THREADS_ENTER ();
 	rb_removable_media_manager_scan (shell->priv->removable_media_manager);
-	GDK_THREADS_LEAVE ();
 
 	if (shell->priv->no_registration == FALSE) {
 		gboolean loaded, scanned;
@@ -1240,6 +1259,18 @@ rb_shell_class_init (RBShellClass *klass)
 							       "Whether or not to disable plugins",
 							       FALSE,
 							       G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	/**
+	 * RBShell:task-list:
+	 *
+	 * The #RBTaskList instance
+	 */
+	g_object_class_install_property (object_class,
+					 PROP_TASK_LIST,
+					 g_param_spec_object ("task-list",
+							      "task list",
+							      "task list",
+							      RB_TYPE_TASK_LIST,
+							      G_PARAM_READABLE));
 
 	/**
 	 * RBShell::visibility-changed:
@@ -1255,7 +1286,7 @@ rb_shell_class_init (RBShellClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBShellClass, visibility_changed),
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__BOOLEAN,
+			      NULL,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
@@ -1275,7 +1306,7 @@ rb_shell_class_init (RBShellClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBShellClass, visibility_changing),
 			      NULL, NULL,
-			      rb_marshal_BOOLEAN__BOOLEAN_BOOLEAN,
+			      NULL,
 			      G_TYPE_BOOLEAN,
 			      2,
 			      G_TYPE_BOOLEAN,
@@ -1297,13 +1328,14 @@ rb_shell_class_init (RBShellClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (RBShellClass, create_song_info),
 			      NULL, NULL,
-			      rb_marshal_VOID__OBJECT_BOOLEAN,
+			      NULL,
 			      G_TYPE_NONE,
 			      2,
 			      RB_TYPE_SONG_INFO, G_TYPE_BOOLEAN);
 	/**
 	 * RBShell::notify-playing-entry:
 	 * @shell: the #RBShell
+	 * @requested: %TRUE if user requested
 	 *
 	 * Emitted when a notification should be displayed showing the current
 	 * playing entry.
@@ -1314,7 +1346,7 @@ rb_shell_class_init (RBShellClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      g_cclosure_marshal_VOID__BOOLEAN,
+			      NULL,
 			      G_TYPE_NONE,
 			      1,
 			      G_TYPE_BOOLEAN);
@@ -1335,7 +1367,7 @@ rb_shell_class_init (RBShellClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      rb_marshal_VOID__UINT_STRING_STRING_STRING_BOOLEAN,
+			      NULL,
 			      G_TYPE_NONE,
 			      5,
 			      G_TYPE_UINT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
@@ -1485,6 +1517,9 @@ rb_shell_get_property (GObject *object,
 	case PROP_DISABLE_PLUGINS:
 		g_value_set_boolean (value, shell->priv->disable_plugins);
 		break;
+	case PROP_TASK_LIST:
+		g_value_set_object (value, shell->priv->task_list);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -1516,11 +1551,8 @@ rb_shell_sync_state (RBShell *shell)
 static gboolean
 idle_save_playlist_manager (RBShell *shell)
 {
-	GDK_THREADS_ENTER ();
 	rb_playlist_manager_save_playlists (shell->priv->playlist_manager,
 					    FALSE);
-	GDK_THREADS_LEAVE ();
-
 	return TRUE;
 }
 
@@ -1616,6 +1648,11 @@ rb_shell_constructed (GObject *object)
 	RBShell *shell;
 	GAction *action;
 	RBEntryView *view;
+	GApplication *app;
+	const char *jump_accels[] = {
+		"<Ctrl>j",
+		NULL
+	};
 
 	/* need this? */
 	gtk_init (NULL, NULL);
@@ -1623,6 +1660,7 @@ rb_shell_constructed (GObject *object)
 	RB_CHAIN_GOBJECT_METHOD (rb_shell_parent_class, constructed, object);
 
 	shell = RB_SHELL (object);
+	app = g_application_get_default ();
 
 	/* construct enough of the rest of it to display the window if required */
 
@@ -1662,26 +1700,19 @@ rb_shell_constructed (GObject *object)
 			 shell->priv->queue_source, "visibility",
 			 G_SETTINGS_BIND_INVERT_BOOLEAN);
 
-	action = g_settings_create_action (shell->priv->settings, "statusbar-visible");
+	action = g_settings_create_action (shell->priv->settings, "follow-playing");
 	g_action_map_add_action (G_ACTION_MAP (shell->priv->window), action);
-	g_settings_bind (shell->priv->settings, "statusbar-visible",
-			 shell->priv->statusbar, "visible",
-			 G_SETTINGS_BIND_DEFAULT);
-
-	action = G_ACTION (g_simple_action_new_stateful ("party-mode",
-							 NULL,
-							 g_variant_new_boolean (FALSE)));
-	g_action_map_add_action (G_ACTION_MAP (shell->priv->window), action);
-	g_signal_connect (action, "activate", G_CALLBACK (view_party_mode_changed_cb), shell);
+	g_signal_connect (shell->priv->settings, "changed", G_CALLBACK (rb_shell_settings_changed_cb), shell);
 
 	action = G_ACTION (g_simple_action_new ("library-import", NULL));
 	g_signal_connect (action, "activate", G_CALLBACK (add_music_action_cb), shell);
-	g_action_map_add_action (G_ACTION_MAP (g_application_get_default ()), action);
+	g_action_map_add_action (G_ACTION_MAP (app), action);
 
 	action = G_ACTION (g_simple_action_new ("jump-to-playing", NULL));
 	g_signal_connect (action, "activate", G_CALLBACK (jump_to_playing_action_cb), shell);
 	g_action_map_add_action (G_ACTION_MAP (shell->priv->window), action);
 
+	gtk_application_set_accels_for_action (GTK_APPLICATION (app), "win.jump-to-playing", jump_accels);
 
 	rb_debug ("shell: syncing with settings");
 
@@ -1696,7 +1727,6 @@ rb_shell_constructed (GObject *object)
 	construct_plugins (shell);
 
 	rb_shell_sync_window_state (shell, FALSE);
-	rb_shell_sync_party_mode (shell);
 
 	rb_shell_select_page (shell, RB_DISPLAY_PAGE (shell->priv->library_source));
 
@@ -1754,7 +1784,6 @@ rb_shell_window_state_cb (GtkWidget *widget,
 						"maximized",
 						maximised);
 		}
-		rb_shell_sync_window_state (shell, TRUE);
 		rb_shell_sync_paned (shell);
 	}
 
@@ -1891,12 +1920,7 @@ rb_shell_window_delete_cb (GtkWidget *win,
 			   GdkEventAny *event,
 			   RBShell *shell)
 {
-	if (shell->priv->party_mode) {
-		return TRUE;
-	}
-
 	rb_shell_quit (shell, NULL);
-
 	return TRUE;
 }
 
@@ -1905,25 +1929,41 @@ rb_shell_key_press_event_cb (GtkWidget *win,
 			     GdkEventKey *event,
 			     RBShell *shell)
 {
-#ifndef HAVE_MMKEYS
-	return FALSE;
-#else
-
-	gboolean retval = TRUE;
+	GtkWindow *window = GTK_WINDOW (win);
+	gboolean handled = FALSE;
 
+#ifdef HAVE_MMKEYS
 	switch (event->keyval) {
 	case XF86XK_Back:
 		rb_shell_player_do_previous (shell->priv->player_shell, NULL);
+		handled = TRUE;
 		break;
 	case XF86XK_Forward:
 		rb_shell_player_do_next (shell->priv->player_shell, NULL);
+		handled = TRUE;
 		break;
 	default:
-		retval = FALSE;
+		break;
+	}
+#endif
+
+	if (!handled)
+		handled = gtk_window_activate_key (window, event);
+
+	if (!handled)
+		handled = gtk_window_propagate_key_event (window, event);
+
+	if (!handled)
+		handled = rb_application_activate_key (shell->priv->application, event);
+
+	if (!handled) {
+		GObjectClass *object_class;
+		object_class = G_OBJECT_GET_CLASS (win);
+		handled = GTK_WIDGET_CLASS (g_type_class_peek_parent (object_class))->key_press_event (win, event);
 	}
 
-	return retval;
-#endif /* !HAVE_MMKEYS */
+	/* we're completely replacing the default window handling, so always return TRUE */
+	return TRUE;
 }
 
 static void
@@ -1995,7 +2035,7 @@ rb_shell_activate_source (RBShell *shell, RBSource *source, guint play, GError *
 		/* fall through */
 	case RB_SHELL_ACTIVATION_ALWAYS_PLAY:
 		rb_shell_player_set_playing_source (shell->priv->player_shell, source);
-		return rb_shell_player_playpause (shell->priv->player_shell, FALSE, error);
+		return rb_shell_player_playpause (shell->priv->player_shell, error);
 
 	default:
 		return FALSE;
@@ -2127,6 +2167,8 @@ rb_shell_display_page_deleted_cb (RBDisplayPage *page, RBShell *shell)
 			rb_shell_player_stop (shell->priv->player_shell);
 		}
 
+		rb_track_transfer_queue_cancel_for_source (shell->priv->track_transfer_queue, source);
+
 		shell->priv->sources = g_list_remove (shell->priv->sources, source);
 	}
 
@@ -2204,23 +2246,44 @@ rb_shell_playing_changed_cb (RBShellPlayer *player, gboolean playing, RBShell *s
 {
 	const char *tooltip;
 	const char *icon_name;
+	GtkWidget *image;
 
+	image = gtk_button_get_image (GTK_BUTTON (shell->priv->play_button));
 	if (playing) {
 		if (rb_source_can_pause (rb_shell_player_get_active_source (shell->priv->player_shell))) {
-			icon_name = "media-playback-pause";
+			icon_name = "media-playback-pause-symbolic";
 			tooltip = _("Pause playback");
 		} else {
-			icon_name = "media-playback-stop";
+			icon_name = "media-playback-stop-symbolic";
 			tooltip = _("Stop playback");
 		}
 	} else {
-		icon_name = "media-playback-start";
+		icon_name = "media-playback-start-symbolic";
 		tooltip = _("Start playback");
 	}
-	gtk_tool_button_set_icon_name (GTK_TOOL_BUTTON (shell->priv->play_button), icon_name);
+	gtk_image_set_from_icon_name (GTK_IMAGE (image), icon_name, GTK_ICON_SIZE_LARGE_TOOLBAR);
+
 	gtk_widget_set_tooltip_text (GTK_WIDGET (shell->priv->play_button), tooltip);
 }
 
+static void
+rb_shell_playing_song_changed_cb (RBShellPlayer *player, RhythmDBEntry *entry, RBShell *shell)
+{
+	if (g_settings_get_boolean (shell->priv->settings, "follow-playing")) {
+		rb_shell_jump_to_current (shell, FALSE);
+	}
+}
+
+static void
+rb_shell_settings_changed_cb (GSettings *settings, const char *key, RBShell *shell)
+{
+	if (g_strcmp0 (key, "follow-playing") == 0) {
+		if (g_settings_get_boolean (settings, key)) {
+			rb_shell_jump_to_current (shell, FALSE);
+		}
+	}
+}
+
 static void
 rb_shell_select_page (RBShell *shell, RBDisplayPage *page)
 {
@@ -2263,7 +2326,6 @@ rb_shell_select_page (RBShell *shell, RBDisplayPage *page)
 
 		/* clear playlist-manager:source? */
 	}
-	rb_statusbar_set_page (shell->priv->statusbar, page);
 
 	g_object_notify (G_OBJECT (shell), "selected-page");
 }
@@ -2318,13 +2380,6 @@ rb_shell_set_window_title (RBShell *shell,
 	}
 }
 
-static void
-view_party_mode_changed_cb (GAction *action, GVariant *parameter, RBShell *shell)
-{
-	shell->priv->party_mode = (shell->priv->party_mode == FALSE);
-	rb_shell_sync_party_mode (shell);
-}
-
 static void
 add_music_action_cb (GAction *action, GVariant *parameter, RBShell *shell)
 {
@@ -2351,10 +2406,8 @@ rb_shell_toggle_visibility (RBShell *shell)
 static gboolean
 quit_timeout (gpointer dummy)
 {
-	GDK_THREADS_ENTER ();
 	rb_debug ("quit damn you");
 	gtk_main_quit ();
-	GDK_THREADS_LEAVE ();
 	return FALSE;
 }
 
@@ -2405,6 +2458,10 @@ rb_shell_quit (RBShell *shell,
 	/* or maybe just _quit */
 	/* g_application_release (G_APPLICATION (shell->priv->application)); */
 
+	/* deselect the current page so it drops mnemonics etc. */
+	rb_display_page_deselected (shell->priv->selected_page);
+
+	rb_settings_delayed_sync (shell->priv->settings, NULL, NULL, NULL);
 	gtk_widget_destroy (GTK_WIDGET (shell->priv->window));
 
 	g_timeout_add_seconds (10, quit_timeout, NULL);
@@ -2414,8 +2471,6 @@ rb_shell_quit (RBShell *shell,
 static gboolean
 idle_handle_load_complete (RBShell *shell)
 {
-	GDK_THREADS_ENTER ();
-
 	rb_debug ("load complete");
 
 	rb_playlist_manager_load_playlists (shell->priv->playlist_manager);
@@ -2438,9 +2493,6 @@ idle_handle_load_complete (RBShell *shell)
 	}
 
 	rhythmdb_start_action_thread (shell->priv->db);
-
-	GDK_THREADS_LEAVE ();
-
 	return FALSE;
 }
 
@@ -2451,64 +2503,6 @@ rb_shell_load_complete_cb (RhythmDB *db,
 	g_idle_add ((GSourceFunc) idle_handle_load_complete, shell);
 }
 
-static gboolean
-window_state_event_cb (GtkWidget           *widget,
-		       GdkEventWindowState *event,
-		       RBShell             *shell)
-{
-	if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED) {
-		rb_shell_present (shell, gtk_get_current_event_time (), NULL);
-	}
-
-	return TRUE;
-}
-
-static void
-rb_shell_sync_party_mode (RBShell *shell)
-{
-	gboolean shell_shows_app_menu = TRUE;
-	GAction *action;
-
-	/* party mode does not use gsettings as a model since it
-	   should not be persistent */
-
-	/* disable/enable quit action */
-	action = g_action_map_lookup_action (G_ACTION_MAP (shell->priv->application), "quit");
-	g_simple_action_set_enabled (G_SIMPLE_ACTION (action), !shell->priv->party_mode);
-
-	/* show/hide menu button */
-	g_object_get (gtk_settings_get_default (),
-		      "gtk-shell-shows-app-menu", &shell_shows_app_menu,
-		      NULL);
-	if (shell_shows_app_menu && (shell->priv->party_mode == FALSE)) {
-		gtk_widget_hide (GTK_WIDGET (shell->priv->menu_button));
-	} else {
-		gtk_widget_show (GTK_WIDGET (shell->priv->menu_button));
-	}
-
-	/* show/hide queue as sidebar ? */
-
-	g_object_set (shell->priv->player_shell, "queue-only", shell->priv->party_mode, NULL);
-
-	/* Set playlist manager source to the current source to update properties */
-	if (shell->priv->selected_page && RB_IS_SOURCE (shell->priv->selected_page)) {
-		RBSource *source = RB_SOURCE (shell->priv->selected_page);
-		g_object_set (shell->priv->playlist_manager, "source", source, NULL);
-		rb_shell_clipboard_set_source (shell->priv->clipboard_shell, source);
-	}
-
-	gtk_window_set_keep_above (GTK_WINDOW (shell->priv->window), shell->priv->party_mode);
-	if (shell->priv->party_mode) {
-		gtk_window_fullscreen (GTK_WINDOW (shell->priv->window));
-		gtk_window_stick (GTK_WINDOW (shell->priv->window));
-		g_signal_connect (shell->priv->window, "window-state-event", G_CALLBACK (window_state_event_cb), shell);
-	} else {
-		gtk_window_unstick (GTK_WINDOW (shell->priv->window));
-		gtk_window_unfullscreen (GTK_WINDOW (shell->priv->window));
-		g_signal_handlers_disconnect_by_func (shell->priv->window, window_state_event_cb, shell);
-	}
-}
-
 static void
 rb_shell_sync_paned (RBShell *shell)
 {
@@ -2555,35 +2549,7 @@ jump_to_playing_action_cb (GSimpleAction *action, GVariant *parameters, gpointer
 {
 	RBShell *shell = RB_SHELL (data);
 	rb_debug ("current song");
-	rb_shell_jump_to_current (shell);
-}
-
-static void
-rb_shell_jump_to_entry_with_source (RBShell *shell,
-				    RBSource *source,
-				    RhythmDBEntry *entry)
-{
-	RBEntryView *songs;
-
-	g_return_if_fail (entry != NULL);
-
-	if ((source == RB_SOURCE (shell->priv->queue_source) &&
-	     g_settings_get_boolean (shell->priv->settings, "queue-as-sidebar")) ||
-	     source == NULL) {
-		RhythmDBEntryType *entry_type;
-		entry_type = rhythmdb_entry_get_entry_type (entry);
-		source = rb_shell_get_source_by_entry_type (shell, entry_type);
-	}
-	if (source == NULL)
-		return;
-
-	songs = rb_source_get_entry_view (source);
-	rb_shell_select_page (shell, RB_DISPLAY_PAGE (source));
-
-	if (songs != NULL) {
-		rb_entry_view_scroll_to_entry (songs, entry);
-		rb_entry_view_select_entry (songs, entry);
-	}
+	rb_shell_jump_to_current (shell, TRUE);
 }
 
 static void
@@ -2591,24 +2557,38 @@ rb_shell_play_entry (RBShell *shell,
 		     RhythmDBEntry *entry)
 {
 	rb_shell_player_stop (shell->priv->player_shell);
-	rb_shell_jump_to_entry_with_source (shell, NULL, entry);
 	rb_shell_player_play_entry (shell->priv->player_shell, entry, NULL);
 }
 
 static void
-rb_shell_jump_to_current (RBShell *shell)
+rb_shell_jump_to_current (RBShell *shell, gboolean select_page)
 {
 	RBSource *source;
-	RhythmDBEntry *playing;
+	RhythmDBEntry *entry;
+	RBEntryView *songs;
 
-	source = rb_shell_player_get_playing_source (shell->priv->player_shell);
+	if (g_settings_get_boolean (shell->priv->settings, "queue-as-sidebar")) {
+		source = rb_shell_player_get_active_source (shell->priv->player_shell);
+	} else {
+		source = rb_shell_player_get_playing_source (shell->priv->player_shell);
+	}
 
-	g_return_if_fail (source != NULL);
+	if (source == NULL)
+		return;
 
-	playing = rb_shell_player_get_playing_entry (shell->priv->player_shell);
 
-	rb_shell_jump_to_entry_with_source (shell, source, playing);
-	rhythmdb_entry_unref (playing);
+	if (select_page) {
+		rb_shell_select_page (shell, RB_DISPLAY_PAGE (source));
+	}
+	songs = rb_source_get_entry_view (source);
+
+	if (songs != NULL) {
+		entry = rb_shell_player_get_playing_entry (shell->priv->player_shell);
+		if (entry != NULL) {
+			rb_entry_view_scroll_to_entry (songs, entry);
+			rhythmdb_entry_unref (entry);
+		}
+	}
 }
 
 void
@@ -2747,6 +2727,7 @@ typedef struct {
 	RBSource *playlist_source;
 	gboolean can_use_playlist;
 	gboolean source_is_entry;
+	gboolean fake_playlist;
 } PlaylistParseData;
 
 static void
@@ -2757,6 +2738,13 @@ handle_playlist_entry_cb (TotemPlParser *playlist,
 {
 	RBSource *source;
 
+	/*
+	 * If we get a fake playlist (single entry containing the input uri),
+	 * we should pretend the file wasn't parsed as a playlist at all.
+	 */
+	if (g_str_equal (uri, data->uri) == FALSE)
+		data->fake_playlist = FALSE;
+
 	/*
 	 * Track whether the same playlist-handling source
 	 * wants all the URIs from the playlist; if it does,
@@ -2838,7 +2826,7 @@ load_uri_parser_finished_cb (GObject *parser, GAsyncResult *res, PlaylistParseDa
 		rb_debug ("%s ignored", data->uri);
 	}
 
-	if (result == TOTEM_PL_PARSER_RESULT_SUCCESS) {
+	if (result == TOTEM_PL_PARSER_RESULT_SUCCESS && data->fake_playlist == FALSE) {
 
 		if (data->can_use_playlist && data->playlist_source) {
 			rb_debug ("adding playlist %s to source", data->uri);
@@ -2950,6 +2938,7 @@ rb_shell_load_uri (RBShell *shell,
 		data->play = play;
 		data->can_use_playlist = TRUE;
 		data->source_is_entry = FALSE;
+		data->fake_playlist = TRUE;
 		data->playlist_source = NULL;
 
 		rb_debug ("adding uri %s, play %d", uri, play);
@@ -2975,20 +2964,6 @@ rb_shell_load_uri (RBShell *shell,
 	return TRUE;
 }
 
-/**
- * rb_shell_get_party_mode:
- * @shell: the #RBShell
- *
- * Returns %TRUE if the shell is in party mode
- *
- * Return value: %TRUE if the shell is in party mode
- */
-gboolean
-rb_shell_get_party_mode (RBShell *shell)
-{
-	return shell->priv->party_mode;
-}
-
 /**
  * rb_shell_present:
  * @shell: the #RBShell
@@ -3070,7 +3045,7 @@ rb_shell_activate_source_by_uri (RBShell *shell,
  * rb_shell_get_song_properties:
  * @shell: the #RBShell
  * @uri: the URI to query
- * @properties: (out callee-allocates) (element-type utf8 GObject.Value) returns the properties of the specified URI
+ * @properties: (out callee-allocates) (element-type utf8 GObject.Value): returns the properties of the specified URI
  * @error: returns error information
  *
  * Gathers and returns all metadata (including extra metadata such as album
