@@ -38,9 +38,7 @@
  * Unlike a GtkTable, the child widgets do not need to align in a grid.
  */
 
-#ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include "eggwrapbox.h"
 #include "eggwrapbox-enums.h"
